<?php

namespace Insign\PostCore\Block\Adminhtml\Order\Barcode;

use Exception;
use Insign\PostCore\Model\Barcode;
use Insign\PostCore\Model\BarcodeFactory;
use Insign\PostCore\Model\BarcodeRepository;
use Magento\Backend\Block\Template;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\App\Response\Http\FileFactory;
use Magento\Framework\Url\EncoderInterface;
use Magento\Framework\UrlInterface;
use Magento\Sales\Model\Order;
use Insign\PostCore\Logger\Log;

class View extends Template
{
    /** @var BarcodeFactory  */
    protected $barcodeFactory;

    /** @var BarcodeRepository  */
    protected $barcodeRepository;

    /** @var UrlInterface  */
    protected $urlInterface;

    /** @var EncoderInterface  */
    protected $urlEncoder;

    /** @var SearchCriteriaBuilder  */
    protected $searchCriteriaBuilder;

    /** @var FilterBuilder  */
    protected $filterBuilder;

    /** FileFactory */
    protected $fileFactory;

    /** @var Log  */
    protected $logger;

    /**
     * View constructor.
     * @param Context $context
     * @param BarcodeFactory $barcodeFactory
     * @param BarcodeRepository $barcodeRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param FilterBuilder $filterBuilder
     * @param UrlInterface $urlInterface
     * @param EncoderInterface $urlEncoder
     * @param FileFactory $fileFactory
     * @param Log $logger
     * @param array $data
     */
    public function __construct(
        Context $context,
        BarcodeFactory $barcodeFactory,
        BarcodeRepository $barcodeRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        FilterBuilder $filterBuilder,
        UrlInterface $urlInterface,
        EncoderInterface $urlEncoder,
        FileFactory $fileFactory,
        Log $logger,
        array $data = []
    ) {
        $this->barcodeFactory = $barcodeFactory;
        $this->barcodeRepository = $barcodeRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;
        $this->urlInterface = $urlInterface;
        $this->urlEncoder = $urlEncoder;
        $this->fileFactory = $fileFactory;
        $this->logger = $logger;
        parent::__construct($context, $data);
    }

    public function getBarcodesByShipmentId(): array
    {
        $shipmentId = $this->getRequest()->getParam('shipment_id');
        $filter = $this->filterBuilder
            ->setField(Barcode::SHIPPING_ID)
            ->setConditionType('eq')
            ->setValue($shipmentId)
            ->create();

        $this->searchCriteriaBuilder->addFilters([$filter]);
        $this->searchCriteriaBuilder->setPageSize(20);
        $searchCriteria = $this->searchCriteriaBuilder->create();

        return $this->barcodeRepository->getList($searchCriteria)->getItems();
    }


    /**
     * Retrieve tracking url with params
     *
     * @param string $key
     * @param Order
     * |\Magento\Sales\Model\Order\Shipment|\Magento\Sales\Model\Order\Shipment\Track $model
     * @param string $method Optional - method of a model to get id
     * @return string
     */
    protected function _getBarcodeUrl(string $key, $model, $method = 'getEntityId'): string
    {
        $urlPart = "{$key}:{$model->{$method}()}:{$model->getProtectCode()}";
        $params = [
            '_scope' => $model->getStoreId(),
            '_nosid' => true,
            '_direct' => 'postc/barcode/popup',
            '_query' => ['hash' => $this->urlEncoder->encode($urlPart)]
        ];

        return $this->urlInterface->getUrl('', $params);
    }

    /**
     * @param $model
     * @return string
     */
    public function getBarcodePopupUrlByBarcodeModel($model): string
    {
        if ($model instanceof Barcode) {
            return $this->_getBarcodeUrl('entity_id', $model);
        }
        return '';
    }

    public function getFilepath($barcode): ?string
    {
        if ($barcode instanceof Barcode) {
            $mediaUrl = substr($this->_storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA), 0, -1);
            return $mediaUrl . $barcode->getPath();
        }
        return "";
    }

    public function isPicture($barcode): bool
    {
        $filetype = strtoupper(pathinfo($barcode->getPath(), PATHINFO_EXTENSION));

        if ($filetype == "GIF" || $filetype == "JPG" || $filetype == "PNG"){
            return true;
        }
        else {
            return false;
        }
    }
}
