<?php

namespace Insign\PostCore\Block\Barcode;

use Exception;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\BarcodeRepository;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;

class Popup extends Template
{
    /**
     * @var Session
     */
    protected $sessionBarcode;
    /**
     * @var BarcodeRepository
     */
    protected $barcodeRepository;
    /**
     * @var UrlInterface
     */
    protected $url;

    /** @var Log  */
    protected $logger;

    /** @var DirectoryList */
    protected $directoryList;

    /**
     * Popup constructor.
     * @param UrlInterface $url
     * @param BarcodeRepository $barcodeRepository
     * @param Session $sessionBarcode
     * @param Template\Context $context
     * @param DirectoryList $directoryList
     * @param Log $logger
     * @param array $data
     */
    public function __construct(
        UrlInterface $url,
        BarcodeRepository $barcodeRepository,
        Session $sessionBarcode,
        Template\Context $context,
        DirectoryList $directoryList,
        Log $logger,
        array $data = []
    ) {
        $this->sessionBarcode = $sessionBarcode;
        $this->barcodeRepository = $barcodeRepository;
        $this->url = $url;
        $this->directoryList = $directoryList;
        $this->logger = $logger;
        parent::__construct($context, $data);
    }

    public function getBase64Barcode(): string
    {
        $barcodeId = $this->sessionBarcode->getVarValue();

        try {
            $barcode = $this->barcodeRepository->getById($barcodeId);
            $mediaPath = $this->directoryList->getPath(DirectoryList::MEDIA);

            if (file_exists($mediaPath . $barcode->getPath())) {
                return base64_encode(file_get_contents($mediaPath . $barcode->getPath()));
            }
        } catch (Exception $e) {
            $this->logger->error("No barcode found for id: " . $barcodeId . "message: " . $e->getMessage());
            throw new Exception("No barcode found for id: " . $barcodeId . " message: " . $e->getMessage());
        }
    }

    public function getTypeImage(): string
    {
        $barcodeId = $this->sessionBarcode->getVarValue();

        try {
            $barcode = $this->barcodeRepository->getById($barcodeId);
            $mediaPath = $this->directoryList->getPath(DirectoryList::MEDIA);
            if (file_exists($mediaPath . $barcode->getPath())) {
                $path = $mediaPath . $barcode->getPath();
                $ext = strtolower(pathinfo($path, PATHINFO_EXTENSION));

                switch ($ext) :
                    case "png":
                    case "eps":
                    case "gif":
                    case "jpg":
                    case "pdf":
                    case "spdf":
                    case "zpl2":
                        return $ext;
                    default:
                        return "image extension does not exist";
                endswitch;
            }
        } catch (Exception $e) {
            $this->logger->error("No image extension found for id: " . $barcodeId . "message: " . $e->getMessage());
            throw new Exception("No image extension found for id: " . $barcodeId . " message: " . $e->getMessage());
        }
    }
}
