<?php

namespace Insign\PostCore\Controller\Adminhtml\Barcode;

use Exception;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\ResourceModel\Barcode\BarcodeCollectionFactory;
use Magento\Backend\App\Action;
use Magento\Checkout\Model\Session;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\View\Result\PageFactory;
use Magento\Ui\Component\MassAction\Filter;

class MassPrint extends Action
{
    /**
     * @var Filter
     */
    private $filter;
    /**
     * @var BarcodeCollectionFactory
     */
    private $collectionFactory;
    /**
     * @var PageFactory
     */
    private $pageFactory;
    /**
     * @var Action\Context
     */
    private $context;
    /**
     * @var Session
     */
    private $barcodeSession;
    /**
     * @var Log
     */
    private $logger;

    /**
     * MassPrint constructor.
     * @param Action\Context $context
     * @param Filter $filter
     * @param BarcodeCollectionFactory $collectionFactory
     * @param PageFactory $pageFactory
     * @param Session $barcodeSession
     * @param Log $logger
     */
    public function __construct(
        Action\Context $context,
        Filter $filter,
        BarcodeCollectionFactory $collectionFactory,
        PageFactory $pageFactory,
        Session $barcodeSession,
        Log $logger
    ) {
        parent::__construct($context);
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        $this->pageFactory = $pageFactory;
        $this->context = $context;
        $this->barcodeSession = $barcodeSession;
        $this->logger = $logger;
    }

    public function execute()
    {
        try {
            $collection = $this->filter->getCollection($this->collectionFactory->create());
            $collectionsArray = [];

            foreach ($collection->getItems() as $item) {
                $barcode = $item->getLabelFilePath();
                array_push($collectionsArray, $barcode);
            }
            $this->massAction($collectionsArray);
            return $this->pageFactory->create();
        } catch (Exception $e) {
            $this->logger->error('Error occurred during getting barcode path' . 'Error message: ' . $e->getMessage());
            return $this->resultFactory->create(ResultFactory::TYPE_REDIRECT)->setPath($this->getUrl('postc/barcode/'));
        }
    }

    /**
     * @param $collection
     */
    protected function massAction($collection)
    {
        $this->barcodeSession->setVarValue($collection);
    }
}
