<?php

namespace Insign\PostCore\Model;

use Exception;
use Insign\PostCore\Helper\BarcodeHelper;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\ResourceModel\Barcode\BarcodeCollectionFactory;
use Magento\Checkout\Model\Session;
use Magento\Sales\Api\ShipmentRepositoryInterface;
use Magento\Sales\Model\OrderRepository;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Ui\DataProvider\AbstractDataProvider;

class DataProvider extends AbstractDataProvider
{
    protected $loadedData;

    /** @var Session  */
    protected $session;

    /** @var BarcodeHelper  */
    protected $barcodeHelper;

    /** @var OrderRepository  */
    protected $orderRepository;

    /** @var ShipmentRepositoryInterface  */
    protected $shipmentRepository;

    /** @var Log  */
    protected $logger;

    /** @var StoreManagerInterface  */
    protected $storeManager;

    /**
     * @param Session $session
     * @param string $name
     * @param string $primaryFieldName
     * @param string $requestFieldName
     * @param BarcodeHelper $barcodeHelper
     * @param BarcodeCollectionFactory $barcodeCollectionFactory
     * @param ShipmentRepositoryInterface $shipmentRepository
     * @param OrderRepository $orderRepository
     * @param Log $logger
     * @param StoreManagerInterface $storeManager
     * @param array $meta
     * @param array $data
     */
    public function __construct(
        Session $session,
        $name,
        $primaryFieldName,
        $requestFieldName,
        BarcodeHelper $barcodeHelper,
        BarcodeCollectionFactory $barcodeCollectionFactory,
        ShipmentRepositoryInterface $shipmentRepository,
        OrderRepository $orderRepository,
        Log $logger,
        StoreManagerInterface $storeManager,
        array $meta = [],
        array $data = []
    ) {
        $this->session = $session;
        $this->barcodeHelper = $barcodeHelper;
        $this->collection = $barcodeCollectionFactory->create();
        $this->shipmentRepository = $shipmentRepository;
        $this->orderRepository = $orderRepository;
        $this->logger = $logger;
        $this->storeManager = $storeManager;
        parent::__construct($name, $primaryFieldName, $requestFieldName, $meta, $data);
    }

    /**
     * Get data
     *
     * @return array
     * @throws Exception
     */
    public function getData(): array
    {
        if (isset($this->loadedData)) {
            return $this->loadedData;
        }

        $shipmentId = $this->session->getVarValue();
        try {
            $shipment = $this->shipmentRepository->get($shipmentId);
            $websiteId = $this->storeManager->getStore($shipment->getStoreId())->getWebsiteId();
        } catch (Exception $e)  {
            $this->logger->error("Can't find shipping: " . $e->getMessage());
            throw new Exception("Can't find shipping: " . $e->getMessage());
        }

        try {
            $order = $this->orderRepository->get($shipment->getOrderId());

        } catch (Exception $e)  {
            $this->logger->error("Can't find order: " . $e->getMessage());
            throw new Exception("Can't find order: " . $e->getMessage());
        }

       $barcodes = $this->collection->getItems();

        foreach ($barcodes as $barcode) {
            $this->loadedData[$barcode->getId()] = $barcode->getData();
        }

        $data['franking_license'] = $this->barcodeHelper->getFrankingLicense($websiteId);
        $data['service_code'] = $this->barcodeHelper->getServiceCode($order->getShippingMethod(), $websiteId);
        $data['additional_service'] = $this->barcodeHelper->getAdditionalService($websiteId);

        if (!empty($data)) {
            $barcode = $this->collection->getNewEmptyItem();
            $barcode->setData($data);
            $this->loadedData[$barcode->getId()] = $barcode->getData();
        }

        return $this->loadedData;
    }
}
