<?php
namespace Insign\PostCore\Model;

use Exception;
use Insign\PostCore\Api\TokenRepositoryInterface;
use Insign\PostCore\Api\Data\TokenInterface;
use Insign\PostCore\Api\Data\TokenSearchResultInterface;
use Insign\PostCore\Api\Data\TokenSearchResultInterfaceFactory;
use Insign\PostCore\Model\ResourceModel\Token\TokenCollectionFactory;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Framework\Exception\NoSuchEntityException;

class TokenRepository implements TokenRepositoryInterface
{
    /** @var TokenFactory  */
    private $tokenFactory;

    /** @var TokenCollectionFactory  */
    private $tokenCollectionFactory;

    /** @var TokenSearchResultInterfaceFactory  */
    private $searchResultFactory;

    /** @var CollectionProcessorInterface */
    private $collectionProcessor;

    /** @var \Insign\PostCore\Model\ResourceModel\Token  */
    private $resource;

    public function __construct(
        TokenFactory $tokenFactory,
        \Insign\PostCore\Model\ResourceModel\Token $resource,
        TokenCollectionFactory $tokenCollectionFactory,
        TokenSearchResultInterfaceFactory $searchResultFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->tokenFactory = $tokenFactory;
        $this->resource = $resource;
        $this->tokenCollectionFactory = $tokenCollectionFactory;
        $this->searchResultFactory = $searchResultFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @param int $id
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getById(int $id)
    {
        $token = $this->tokenFactory->create();
        $this->resource->load($token, $id);
        if (! $token->getId()) {
            throw new NoSuchEntityException(__('Unable to find token with ID "%1"', $id));
        }
        return $token;
    }

    /**
     * @param TokenInterface $token
     * @return TokenInterface
     * @throws AlreadyExistsException
     */
    public function save(TokenInterface $token): TokenInterface
    {
        $token->getResource()->save($token);
        return $token;
    }

    /**
     * @param TokenInterface $token
     * @throws Exception
     */
    public function delete(TokenInterface $token)
    {
        $token->getResource()->delete($token);
    }


    /**
     * @param SearchCriteriaInterface $searchCriteria
     * @return TokenSearchResultInterface
     */
    public function getList(SearchCriteriaInterface $searchCriteria): TokenSearchResultInterface
    {
        $collection = $this->tokenCollectionFactory->create();
        $this->collectionProcessor->process($searchCriteria, $collection);
        $searchResults = $this->searchResultFactory->create();

        $searchResults->setSearchCriteria($searchCriteria);
        $searchResults->setItems($collection->getItems());

        return $searchResults;
    }
}
